<?php

declare(strict_types=1);

namespace mirzaev\parser_from_interneturok\models\telegram;

// Files of the project
use mirzaev\parser_from_interneturok\models\core,
	mirzaev\parser_from_interneturok\models\interneturok,
	mirzaev\parser_from_interneturok\models\enumerations\subject;

// Framework for Telegram
use Zanzara\Context as context,
	Zanzara\Telegram\Type\Message as message,
	Zanzara\Telegram\Type\Input\InputFile as file_input;

// Framework for asynchronous PHP
use function React\Async\await;

/**
 * Telegram shcool subjects parser
 *
 * @package mirzaev\parser_from_interneturok\models\telegram
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class parser extends core
{
	/**
	 * Process
	 *
	 * @var const string PROCESS Name of the process in the telegram user buffer
	 */
	public const string PROCESS = 'parse_subject';

	/**
	 * Subject
	 *
	 * Create the process, write subject to the buffer and request grade
	 *
	 * @param context $context Request data from Telegram
	 * @param subject $subject The schrool subject
	 *
	 * @return void
	 */
	public static function subject(context $context, subject $subject): void
	{
		// Initializing the parsing target buffer
		$target = [
			'subject' => $subject,
			'grade' => null
		];

		// Writing to the telegram user buffer
		$context->setUserDataItem(static::PROCESS, $target)
			->then(function () use ($context) {
				// Writed to the telegram user buffer

				// Sending the message
				$context->sendMessage(
					'🎒 *Выбери номер класса*',
					[
						'reply_markup' => [
							'inline_keyboard' => [
								[
									[
										'text' => '4',
										'callback_data' => 'grade_4'
									],
									[
										'text' => '5',
										'callback_data' => 'grade_5'
									],
									[
										'text' => '6',
										'callback_data' => 'grade_6'
									],
									[
										'text' => '7',
										'callback_data' => 'grade_7'
									]
								],
								[
									[
										'text' => '8',
										'callback_data' => 'grade_8'
									],
									[
										'text' => '9',
										'callback_data' => 'grade_9'
									],
									[
										'text' => '10',
										'callback_data' => 'grade_10'
									],
									[
										'text' => '11',
										'callback_data' => 'grade_11'
									]
								]
							],
							'disable_notification' => true,
							'remove_keyboard' => true
						]
					]
				);
			});
	}

	/**
	 * Grade
	 *
	 * Write grade to the buffer and request confirmation for starting the parsing process
	 *
	 * @param context $context Request data from Telegram
	 * @param int $grade The grade
	 *
	 * @return void
	 */
	public static function grade(context $context, int $grade): void
	{
		// Reading from the telegram user buffer
		$context->getUserDataItem(static::PROCESS)
			->then(function (?array $target) use ($context, $grade) {
				// Readed from the telegram user buffer

				if ($target) {
					// Initialized the parsing process

					// Initializing the grade
					$target['grade'] = $grade;

					// Writing to the telegram user buffer
					$context->setUserDataItem(static::PROCESS, $target)
						->then(function () use ($context, $target) {
							// Writed to the telegram user buffer

							// Initializing the subject
							$subject = $target['subject']?->value;

							// Initializing the grade
							$grade = $target['grade'];

							// Sending the message
							$context->sendMessage(
								<<<TXT
								📦 *Генерация запроса*

								*Предмет:* $subject
								*Класс:* $grade
								TXT,
								[
									'reply_markup' => [
										'inline_keyboard' => [
											[
												[
													'text' => '✅ Отправить запрос',
													'callback_data' => 'parse'
												]
											]
										],
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							);
						});
				} else {
					// Not initialized the parsing process

					// Sending the message
					$context->sendMessage('⚠️ *Не запущен процесс генерации парсинга*')
						->then(function (message $message) use ($context) {
							// Sended the message

							// Sending the menu with subjects
							commands::menu($context);
						});
				}
			});
	}

	/**
	 * Parse
	 *
	 * @param context $context Request data from Telegram
	 *
	 * @return void
	 */
	public static function parse(context $context): void
	{
		// Reading from the telegram user buffer
		$context->getUserDataItem(static::PROCESS)
			->then(function (?array $target) use ($context) {
				// Readed from the telegram user buffer

				if ($target) {
					// Initialized the parsing process

					// Initializing variables for the parsing process
					$subject = $target['subject'];
					$grade = $target['grade'];

					// Initializing the parser
					$parser = new interneturok();

					// Parsing homework files
					$files = $parser->parse(subject: $subject, grade: $grade, waiting: 0);

					if (!empty($files)) {
						// Initialized homework files

						foreach ($files as $file) {
							// Iterating over files in the storage

							// Sending the file
							await($context->sendDocument(new file_input($file)));
						}

						// Deleting in the telegram user buffer
						$context->deleteUserDataItem(static::PROCESS);
					} else {
						// Not initialized homework files

						// Sending the message
						$context->sendMessage('❌ *Не найдены домашние задания')
							->then(function (message $message) use ($context) {
								// Sended the message

								// Sending the menu with subjects
								commands::menu($context);
							});
					}
				} else {
					// Not initialized the parsing process

					// Sending the message
					$context->sendMessage('⚠️ *Не запущен процесс генерации парсинга*')
						->then(function (message $message) use ($context) {
							// Sended the message

							// Sending the menu with subjects
							commands::menu($context);
						});
				}
			});
	}
}
