<?php

declare(strict_types=1);

namespace mirzaev\parser_from_interneturok;

// Framework for Telegram
use Zanzara\Zanzara as zanzara,
	Zanzara\Context as context,
	Zanzara\Config as config;

// Files of the project
use mirzaev\parser_from_interneturok\models\telegram\commands,
	mirzaev\parser_from_interneturok\models\telegram\middlewares,
	mirzaev\parser_from_interneturok\models\telegram\parser,
	mirzaev\parser_from_interneturok\models\enumerations\subject;

// Enabling debugging
/* ini_set('error_reporting', E_ALL);
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1); */

// Initializing path to the public directory 
define('INDEX', __DIR__);

// Initializing path to the project root directory
define('ROOT',  INDEX . DIRECTORY_SEPARATOR	. '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR	. '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR);

// Initializing path to the directory of views 
define('VIEWS', INDEX . DIRECTORY_SEPARATOR	. '..' . DIRECTORY_SEPARATOR . 'views');

// Initializing path to the directory of settings 
define('SETTINGS', INDEX . DIRECTORY_SEPARATOR	. '..' . DIRECTORY_SEPARATOR . 'settings');

// Initializing system settings 
require SETTINGS . DIRECTORY_SEPARATOR . 'system.php';

// Initializing path to the directory of the storage 
define('STORAGE', INDEX . DIRECTORY_SEPARATOR	. '..' . DIRECTORY_SEPARATOR . 'storage');

// Initializing path to the databases directory
define('DATABASES', INDEX . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'databases');

// Initiailizing telegram key
define('TELEGRAM_KEY', require(SETTINGS . DIRECTORY_SEPARATOR . 'telegram.php'));

// Initializing dependencies
require ROOT . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

// Initiailizing interneturok accounts
define('INTERNETUROK_ACCOUNTS_FILE', STORAGE . DIRECTORY_SEPARATOR . 'accounts.csv');

// Initiailizing telegram authorized accounts
define('ACCOUNTS', require(SETTINGS . DIRECTORY_SEPARATOR . 'accounts.php'));

// Initializing the configuration
$config = new config();
$config->setParseMode(config::PARSE_MODE_MARKDOWN);
$config->useReactFileSystem(true);

// Initializing the robot
$robot = new Zanzara(TELEGRAM_KEY, $config);

// Initializing the robot middlewares
$robot->middleware([middlewares::class, 'authorization']);

// Initializing the robot commands handlers
$robot->onCommand('start', [commands::class, 'menu']);
$robot->onCommand('menu', [commands::class, 'menu']);

// Initializing the robot subjects buttons handlers
foreach (subject::cases() as $subject) {
	// Iterating over subjects

	// Initializing subjects buttons
	$robot->onCbQueryData([$subject->name], fn(context $context) => parser::subject(context: $context, subject: $subject));
};

for ($i = 4; $i <= 11; ++$i) {
	// Generating buttons from 4 to 11 grades

	// Initializing grades buttons
	$robot->onCbQueryData(["grade_$i"], fn(context $context) => parser::grade(context: $context, grade: $i));
}

// Initializing the parse button
$robot->onCbQueryData(['parse'], [parser::class, 'parse']);

// Initializing the exceptions handler
$robot->onException(function (context $context, $exception) {
	var_dump($exception);
});

// Starting chat-robot
$robot->run();
